C
C SIMULATION OF LONGITUDINAL MUSR DATA.
C FIELD DISTRIBUTION FOR STATIC NUCLEAR DIPOLES WITH HOPPING MUON
C
C DECLARATIONS
C
      subroutine DLKT(PARS_IN,PARS_OUT)
	implicit none
	external pars_in,pars_out

	REAL GAUSS,EXPONENTIAL2
	REAL LORENTZ
	DOUBLE PRECISION COMP1,COMP2,COMP3
	REAL PW,PW3,PW6,PWS
	REAL HW,HBW,HTOT,HAVE
	REAL TAU,D,SIND3,TBW
	REAL TD,TZ,T,TH,TT,TR, TP
	DOUBLE PRECISION PI,HPI,PIS,GAM
	DOUBLE PRECISION HX,HY,HZ,HXS,HYS,HZS,HS,H,HL
	DOUBLE PRECISION PSI,THI,WT
	DOUBLE PRECISION MX,MY,MZ,MXX,MYX,MZX,MXY,MYY,MZY,MXZ,MYZ,MZZ,M
	REAL FPROB,BPROB
	REAL RFWD,RBCK,ASYM,E
	INTEGER N,I,TBIN,HBIN,F,B,COUNT,J
	INTEGER TFLAG,HCOUNT
	INTEGER FWD(2000),BCK(2000),FLD(5000), statusi, runno
	CHARACTER*5 OUTFILE
	CHARACTER*10 DATFILE,LOGFILE,FLDFILE
	CHARACTER*28 txt
	REAL XPT(2000),YPT(2000),YERR(2000)
      REAL XFLD(5000),YFLD(5000),YERRFLD(5000)
 

C
C INPUT DATA FROM OPENGENIE
C
C	WRITE(*,*) 'Enter the number of required events '
	call module_get_int(pars_in,'N',N)
C	WRITE(*,*) 'Enter the muon pulse width (micro secs) '
	call module_get_real(pars_in,'PW',PW)
c	WRITE(*,*) 'Enter the field distribution width (Gauss) '
	call module_get_real(pars_in,'HW',HW)
C	WRITE(*,*) 'Enter the applied longitudinal field (Gauss) '
	call module_get_real(pars_in,'HL',HL)
C	WRITE(*,*) 'Enter the angle subtended by the detector (Rad) '
	call module_get_real(pars_in,'D',D)
C	WRITE(*,*) 'Enter the output filenumber (5 numbers max, no extention) '
	call module_get_int(pars_in,'RUNNO',RUNNO)
c	WRITE(*,*) 'Enter the muon site residence time (micro secs) '
	call module_get_real(pars_in,'TH',TH)
 	WRITE(99,*)N
      WRITE(99,*)PW
      WRITE(99,*)HW
      WRITE(99,*)HL
      WRITE(99,*)D
      WRITE(99,*)RUNNO

      WRITE(OUTFILE,5) RUNNO
	WRITE(99,*) OUTFILE

	WRITE(DATFILE,6) OUTFILE
	WRITE(LOGFILE,7) OUTFILE
	WRITE(FLDFILE,8) OUTFILE
5	FORMAT(I5)
6	FORMAT('R',A5,'.DAT')
7	FORMAT('R',A5,'.LOG')
8	FORMAT('F',A5,'.DAT')
C
C DEFINITIONS
C
c	I=9565
      call system_clock(i)
	statusi=0
      PI=4D0*ATAN(1D0)
      HPI=PI/2D0
	PIS=PI*PI
      GAM=2D0*PI*13.55342D-03
	TAU=2.19714
	TBW=0.016
	TZ=4.0
	SIND3=(SIN(D))/3.0
      PW3=3.0*PW
	PW6=6.0*PW
	PWS=PW*PW
      HBW=SQRT(3.0)*(50.0*HW)/1000
	COUNT=0
	DO j=1,2000
	 FWD(J)=0.0
	 BCK(J)=0.0
	enddo

	DO j=1,5000
       FLD(J)=0.0
	ENDDO
C
C MAIN LOOP
C
C
C GENERATE MUON PULSE DISTRIBUTION
C
10	TP=GAUSS(I,PW6,PW3,PWS)
C
C GENERATE MUON LIFETIME DISTRIBUTION
C
20	TD=EXPONENTIAL2(I,TAU)
	IF (TD+TP.GT.28) THEN
	  GOTO 20
	ENDIF
C
C GENERATE INITIAL MUON SITE RESIDENCE TIME
C
	TR=EXPONENTIAL2(I,TH)
	IF (TR.GT.TD) THEN
	  TR=TD
	  TFLAG=1
	ENDIF
	TT=TR
C
C GENERATE INITIAL FIELD DISTRIBUTION
C
	HX=LORENTZ(I,HW,PI)
      HY=LORENTZ(I,HW,PI)
      HZ=HL+LORENTZ(I,HW,PI)
	HXS=HX*HX
	HYS=HY*HY
	HZS=HZ*HZ
	HS=(HXS+HYS+HZS)
	H=SQRT(HS)
	WT=H*GAM*TR
!        WRITE(*,*)H,HX,HY,HZ
C
C STORE THE INITIAL FIELD MAGNITUDE
C
	HTOT=H
        HCOUNT=1
C
C CALCULATE INITIAL MUON SPIN COMPONENTS IN THE DETECTOR FRAME
C
25	PSI=ACOS(HZ/H)
	IF (HX.EQ.0D0) THEN
	  THI=HPI*HY/(ABS(HY))
	ELSE
	  THI=ATAN(HY/HX)
	ENDIF
	IF (HY.GT.0D0.AND.HZ.LT.0D0) THEN
	  THI=THI+PI
	ENDIF
	IF (HY.LT.0D0.AND.HZ.LT.0D0) THEN
	  THI=THI-PI
	ENDIF
	MX=COMP1(PSI,THI,WT)
	MY=COMP2(PSI,THI,WT)
	MZ=COMP3(PSI,WT)
C
C ASK IF THE MUON HAS DECAYED
C
	IF (TFLAG.EQ.1) THEN
	  TFLAG=0
	  GOTO 40
	ENDIF
C
C HOPPING LOOP
C
C
C GENERATE MUON SITE RESIDENCE TIME
C
30      TR=EXPONENTIAL2(I,TH)
        IF ((TT+TR).GT.TD) THEN
          TR=TD-TT
          TFLAG=1
        ENDIF
	TT=TT+TR
C
C GENERATE NEW FIELD COMPONENTS
C
	HX=LORENTZ(I,HW,PI)
        HY=LORENTZ(I,HW,PI)
        HZ=HL+LORENTZ(I,HW,PI)
        HXS=HX*HX
        HYS=HY*HY
        HZS=HZ*HZ
        HS=(HXS+HYS+HZS)
        H=SQRT(HS)
	WT=H*GAM*TR
!        WRITE(6,*)H,HX,HY,HZ
C
C STORE THE NEW FIELD MAGNITUDE
C	
	HTOT=HTOT+H
	HCOUNT=HCOUNT+1
C
C CALCULATE THE NEW MUON SPIN COMPONENTS FROM THE CURRENT X-SPIN COMPONENT
C
	PSI=ACOS(HX/H)
	IF (HY.EQ.0D0) THEN
	  THI=HPI*HZ/(ABS(HZ))
	ELSE
	  THI=ATAN(HZ/HY)
	ENDIF
	IF (HZ.GT.0D0.AND.HY.LT.0D0) THEN
	  THI=THI+PI
	ENDIF
	IF (HZ.LT.0D0.AND.HY.LT.0D0) THEN
	  THI=THI-PI
	ENDIF
	MYX=MX*COMP1(PSI,THI,WT)
	MZX=MX*COMP2(PSI,THI,WT)
	MXX=MX*COMP3(PSI,WT)
C
C CALCULATE THE NEW MUON SPIN COMPONENTS FROM THE CURRENT Y-SPIN COMPONENT
C
	PSI=ACOS(HY/H)
	IF (HZ.EQ.0D0) THEN
	  THI=HPI*HX/(ABS(HX))
	ELSE
	  THI=ATAN(HX/HZ)
	ENDIF
	IF (HX.GT.0D0.AND.HZ.LT.0D0) THEN
	  THI=THI+PI
	ENDIF
	IF (HX.LT.0D0.AND.HZ.LT.0D0) THEN
	  THI=THI-PI
	ENDIF
	MZY=MY*COMP1(PSI,THI,WT)
	MXY=MY*COMP2(PSI,THI,WT)
	MYY=MY*COMP3(PSI,WT)
C
C CALCULATE THE NEW MUON SPIN COMPONENTS FROM THE CURRENT Z-SPIN COMPONENT
C
	PSI=ACOS(HZ/H)
	IF (HX.EQ.0D0) THEN
	  THI=HPI*HY/(ABS(HY))
	ELSE
	  THI=ATAN(HY/HX)
	ENDIF
	IF (HY.GT.0D0.AND.HX.LT.0D0) THEN
	  THI=THI+PI
	ENDIF
	IF (HY.LT.0D0.AND.HX.LT.0D0) THEN
	  THI=THI-PI
	ENDIF
	MXZ=MZ*COMP1(PSI,THI,WT)
	MYZ=MZ*COMP2(PSI,THI,WT)
	MZZ=MZ*COMP3(PSI,WT)
C
C CALCULATE THE THREE NEW ORTHOGONAL SPIN COMPONENTS
C
	MX=MXX+MXY+MXZ
	MY=MYX+MYY+MYZ
	MZ=MZX+MZY+MZZ
C
C ASK IF THE MUON HAS DECAYED
C
        IF (TFLAG.EQ.1) THEN
	  TFLAG=0
	  GOTO 40
	ELSE
	  GOTO 30
	ENDIF
C
C PROBABILITY OF DETECTING THE POSITRON
C
40	FPROB=D/PIS*(D+MZ*SIND3)
	BPROB=D/PIS*(D-MZ*SIND3)
	TBIN=JNINT((TD+TP+TZ)/TBW)
	F=JNINT(FPROB*10000)
	B=JNINT(BPROB*10000)
	FWD(TBIN)=FWD(TBIN)+F
	BCK(TBIN)=BCK(TBIN)+B
	COUNT=COUNT+F+B
	IF (COUNT.GT.STATUSI*1000000) then
	 WRITE(txt,1972) STATUSI*1000000
1972	 FORMAT('Muons Simulated ',I10)

       CALL MODULE_PRINT(TRIM(txt))
	 STATUSI=STATUSI+1
	endif
C
C STORE THE FIELD AVERAGE FOR THIS MUON
C
	HAVE=HTOT/HCOUNT
	IF (HAVE.GT.(433.0*HW)) GOTO 45
        HBIN=JINT(HAVE/HBW)+1
      	IF (HBIN.LT.4999)   FLD(HBIN)=FLD(HBIN)+1
C
C END OF MAIN LOOP
C
45	IF (COUNT.LT.N) GOTO 10
C
C CALCULATE THE ASYMMETRY PLOT AND WRITE THE FILES
C
	OPEN(2,FILE=DATFILE)
	DO 50 J=1,2000
	T=REAL(J*TBW)-TZ
	IF (FWD(J).EQ.0.OR.BCK(J).EQ.0) THEN
       ASYM=0.0
	 E=0.0
	 WRITE(2,*) T,asym,E
	else
	 RFWD=REAL(FWD(J))
	 RBCK=REAL(BCK(J))
	 ASYM=(RFWD-RBCK)/(RFWD+RBCK)
	 E=SQRT(4.0*RFWD*RBCK*(RFWD+RBCK))/((RFWD+RBCK)**2.0)
	 WRITE(2,*) T,ASYM,E
	ENDIF
	xpt(j)=t
	ypt(j)=asym
	yerr(j)=e
50	CONTINUE
	CLOSE(2)
	OPEN(3,FILE=LOGFILE)
	WRITE(3,*) 'FILE = ',OUTFILE
	WRITE(3,*) 'COUNTS = ',COUNT
        WRITE(3,*) 'PULSE WIDTH = ',PW
	WRITE(3,*) 'FIELD WIDTH = ',HW
	WRITE(3,*) 'LNG FIELD = ',HL
	WRITE(3,*) 'SITE TIME = ',TH
	WRITE(3,*) 'DETECTOR ANGLE = ',D
	CLOSE(3)
	OPEN(4,FILE=FLDFILE)
	DO 60 J=1,2000
	H=REAL(J*HBW)
	WRITE(4,*) H,FLD(J)
	XFLD(J)=H
	YFLD(J)=REAL(FLD(J))
	YERRFLD(J)=E
60	CONTINUE
	CLOSE(4)
	YFLD(1)=0.0
      CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"XFLD",XFLD,5000)
	CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"YFLD",YFLD,5000)
	CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"YERRFLD",YERRFLD,5000)
	CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"XPT",XPT,2000)
	CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"YPT",YPT,2000)
	CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"YERR",YERR,2000)

	END
!
! FUNCTION TO CALCULATE THE FIRST MUON SPIN COMPONENT
!
	FUNCTION COMP1(PSI,THI,WT)
	DOUBLE PRECISION PSI,THI,WT,COMP1
	COMP1=0.5*DSIN(2.0*PSI)*DCOS(THI)*(1.0-DCOS(WT))+DSIN(PSI)*DSIN(THI)*DSIN(WT)
	RETURN
	END

! FUNCTION TO CALCULATE THE SECOND MUON SPIN COMPONENT
!
	FUNCTION COMP2(PSI,THI,WT)
	DOUBLE PRECISION PSI,THI,WT,COMP2
	COMP2=0.5*DSIN(2.0*PSI)*DSIN(THI)*(1.0-DCOS(WT))-DSIN(PSI)*DCOS(THI)*DSIN(WT)
	RETURN
	END
!
! FUNCTION TO CVALCULATE THE THIRD MUON SPIN COMPONENT
!
	FUNCTION COMP3(PSI,WT)
	DOUBLE PRECISION PSI,WT,COMP3
	COMP3=(DCOS(PSI))**2.0+DCOS(WT)*(DSIN(PSI))**2.0
	RETURN
	END

! FUNCTION TO GENERATE EXPONENTIAL PROBABILITY DISTRIBUTION
!
	FUNCTION EXPONENTIAL2(I,TAU)
	REAL X, exponential2,TAU
	INTEGER I
200     X=RAN(I)
        IF (X.EQ.0) THEN
          GOTO 200
        ELSE
          EXPONENTIAL2=-TAU*ALOG(1.0-X)
        ENDIF
	RETURN
	END


! FUNCTION TO GENERATE LORENTZIAN PROBABILITY DISTRIBUTION
!
	FUNCTION LORENTZ(I,HW,PI)
	REAL HW,PI,X,LORENTZ
	INTEGER I
300	X=RAN(I)
	IF (X.EQ.0.5) THEN
	  GOTO 300
	ELSE
	  LORENTZ=HW*TAN(X*PI)
	ENDIF
	RETURN
	END

	FUNCTION GAUSS(I,W,M,WS)
	REAL GAUSS,W,M,WS,X,Y
	INTEGER I
100     X=RAN(I)*W
        Y=EXP(-0.5*(X-M)**2.0/WS)
        IF (RAN(I).LE.Y) THEN
          GAUSS=X-M
        ELSE
          GOTO 100
        ENDIF
	RETURN
	END
