C
C SIMULATION OF LONGITUDINAL MUSR DATA.
C FIELD DISTRIBUTION FOR STATIC DILUTE ATOMIC MOMENTS
C
C DECLARATIONS
C
      subroutine SLKT(PARS_IN,PARS_OUT)

	implicit none
	external pars_in,pars_out

	REAL GAUSS,EXPONENTIAL,LORENTZ
	REAL PW,PW3,PW6,PWS
	REAL HW
	REAL GAM,PI,HPI,PIS,D,SIND3
	REAL TBW,TP,TD,TZ,T
	REAL HL,HX,HY,HZ,H,HXS,HYS,HZS,HS,HBW
	REAL MZ,COSSTHI,SINSTHI,COSWT
	REAL FPROB,BPROB
	REAL RFWD,RBCK,ASYM,E
	INTEGER N,I,TBIN,HBIN,F,B,COUNT,J
	INTEGER FWD(2000),BCK(2000),FLD(0:5000), statusi, runno
	CHARACTER*5 OUTFILE
	CHARACTER*10 DATFILE,LOGFILE,FLDFILE
	CHARACTER*28 txt
      REAL XPT(2000),YPT(2000),YERR(2000)
      REAL XFLD(5000),YFLD(5000),YERRFLD(5000)


C
C INPUT DATA FROM OPENGENIE
C
C	WRITE(*,*) 'Enter the number of required events '
	call module_get_int(pars_in,'N',N)
C	WRITE(*,*) 'Enter the muon pulse width (micro secs) '
	call module_get_real(pars_in,'PW',PW)
c	WRITE(*,*) 'Enter the field distribution width (Gauss) '
	call module_get_real(pars_in,'HW',HW)
C	WRITE(*,*) 'Enter the applied longitudinal field (Gauss) '
	call module_get_real(pars_in,'HL',HL)
C	WRITE(*,*) 'Enter the angle subtended by the detector (Rad) '
	call module_get_real(pars_in,'D',D)
C	WRITE(*,*) 'Enter the output filenumber (5 numbers max, no extention) '
	call module_get_int(pars_in,'RUNNO',RUNNO)
 	WRITE(99,*)N
      WRITE(99,*)PW
      WRITE(99,*)HW
      WRITE(99,*)HL
      WRITE(99,*)D
      WRITE(99,*)RUNNO

      WRITE(OUTFILE,5) RUNNO
	WRITE(99,*) OUTFILE
	WRITE(DATFILE,6) OUTFILE
	WRITE(LOGFILE,7) OUTFILE
	WRITE(FLDFILE,8) OUTFILE
5	FORMAT(I5)
6	FORMAT('R',A5,'.DAT')
7	FORMAT('R',A5,'.LOG')
8	FORMAT('F',A5,'.DAT')
C
C DEFINITIONS
C
C	I=9565
      call system_clock(i)
	statusi=0
        PI=4.0*ATAN(1.0)
        HPI=PI/2.0
	PIS=PI*PI
        GAM=2.0*PI*13.55342E-03
	TBW=0.016
	TZ=4.0
	SIND3=(SIN(D))/3.0
        PW3=3.0*PW
	PW6=6.0*PW
	PWS=PW*PW
	HBW=SQRT(3.0)*(50.0*HW)/1000
	COUNT=0
	DO j=1,2000
	 FWD(J)=0.0
	 BCK(J)=0.0
	enddo
	DO j=0,5000
       FLD(J)=0.0
	ENDDO
C
C MAIN LOOP
C
C
C GENERATE MUON PULSE DISTRIBUTION
C
10	TP=GAUSS(I,PW6,PW3,PWS)
C
C GENERATE MUON LIFETIME DISTRIBUTION
C
20	TD=EXPONENTIAL(I)
	IF (TD+TP.GT.28) THEN
	    GOTO 20
	ENDIF
C
C GENERATE FIELD DISTRIBUTION
C
	HX=LORENTZ(I,HW,PI)
        HY=LORENTZ(I,HW,PI)
        HZ=LORENTZ(I,HW,PI)
	HZ=HZ+HL
	HXS=HX*HX
	HYS=HY*HY
	HZS=HZ*HZ
	HS=(HXS+HYS+HZS)
	H=SQRT(HS)
C
C STORE FIELD DISTRIBUTION
C
	IF (H.GT.(433.0*HW)) GOTO 25
	HBIN=JINT(H/HBW)
	IF (HBIN.LT.4999) FLD(HBIN)=FLD(HBIN)+1
C
C CALCULATE THE MUON SPIN Z COMPONENT
C
25	COSSTHI=HZS/HS
	SINSTHI=1.0-COSSTHI
	COSWT=COS(H*GAM*TD)
	MZ=COSSTHI+SINSTHI*COSWT
C
C PROBABILITY OF DETECTING THE POSITRON
C
	FPROB=D/PIS*(D+MZ*SIND3)
	BPROB=D/PIS*(D-MZ*SIND3)
	TBIN=JINT((TD+TP+TZ)/TBW)
	F=JNINT(FPROB*10000)
	B=JNINT(BPROB*10000)
	FWD(TBIN)=FWD(TBIN)+F
	BCK(TBIN)=BCK(TBIN)+B
	COUNT=COUNT+F+B
	IF (COUNT.GT.STATUSI*1000000) then
	 WRITE(txt,1972) STATUSI*1000000
1972	 FORMAT('Muons Simulated ',I10)

       CALL MODULE_PRINT(TRIM(txt))
	 STATUSI=STATUSI+1
	endif
C
C
C END OF MAIN LOOP
C
	IF (COUNT.LT.N) GOTO 10
C
C CALCULATE THE ASYMMETRY PLOT AND WRITE THE FILES
C
	OPEN(2,FILE=DATFILE)
	DO 30 J=1,2000
		T=REAL(J*TBW)-TZ
		IF (FWD(J).EQ.0.OR.BCK(J).EQ.0) then
			ASYM=0.0
			E=0.0
			WRITE(2,*) T,ASYM,E
		else
			RFWD=REAL(FWD(J))
			RBCK=REAL(BCK(J))
			ASYM=(RFWD-RBCK)/(RFWD+RBCK)
			E=SQRT(4.0*RFWD*RBCK*(RFWD+RBCK))/((RFWD+RBCK)**2.0)
			WRITE(2,*) T,ASYM,E
		ENDIF
		xpt(j)=t
		ypt(j)=asym
		yerr(j)=e
30	CONTINUE
	CLOSE(2)
	OPEN(3,FILE=LOGFILE)
	WRITE(3,*) 'FILE = ',OUTFILE
	WRITE(3,*) 'COUNTS = ',COUNT
      WRITE(3,*) 'PULSE WIDTH = ',PW
	WRITE(3,*) 'FIELD WIDTH = ',HW
	WRITE(3,*) 'LNG FIELD = ',HL
	WRITE(3,*) 'DETECTOR ANGLE = ',D
	CLOSE(3)
	OPEN(4,FILE=FLDFILE)
	DO 40 J=1,5000
		H=REAL(J*HBW)
		E=SQRT(REAL(FLD(J)))
		WRITE(4,*) H,FLD(J),E
		XFLD(J)=H
		YFLD(J)=REAL(FLD(J))
		YERRFLD(J)=E
40	CONTINUE
	CLOSE(4)
      CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"XFLD",XFLD,5000)
	CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"YFLD",YFLD,5000)
	CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"YERRFLD",YERRFLD,5000)
	CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"XPT",XPT,2000)
	CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"YPT",YPT,2000)
	CALL MODULE_PUT_REAL_ARRAY(PARS_OUT,"YERR",YERR,2000)

	END
C
C
C
C
! FUNCTION TO GENERATE GAUSSIAN PROBABILITY DISTRIBUTION
!
	FUNCTION GAUSS(I,W,M,WS)
	REAL GAUSS,W,M,WS,X,Y
	INTEGER I
100     X=RAN(I)*W
        Y=EXP(-0.5*(X-M)**2.0/WS)
        IF (RAN(I).LE.Y) THEN
          GAUSS=X-M
        ELSE
          GOTO 100
        ENDIF
	RETURN
	END
!
!
! FUNCTION TO GENERATE EXPONENTIAL PROBABILITY DISTRIBUTION
!
	FUNCTION EXPONENTIAL(I)
	REAL X,TAU
	INTEGER I
	TAU=2.19714
200     X=RAN(I)
        IF (X.EQ.0) THEN
          GOTO 200
        ELSE
          EXPONENTIAL=-TAU*ALOG(1.0-X)
        ENDIF
	RETURN
	END

! FUNCTION TO GENERATE LORENTZIAN PROBABILITY DISTRIBUTION
!
	FUNCTION LORENTZ(I,HW,PI)
	REAL HW,PI,X,LORENTZ
	INTEGER I
300	X=RAN(I)
	IF (X.EQ.0.5) THEN
	  GOTO 300
	ELSE
	  LORENTZ=HW*TAN(X*PI)
	ENDIF
	RETURN
	END
