c UDAFUNC.FOR, model function calculation
c   complete re-write 3/9 1988
c   major modifications 7/9 1988, more work needed!!
c 
c 12-APR-1995  S P Cottrell  Definition of 'fpulse' changed, now
c                            '0' - Single (old, 'Average')
c                            '1' - Average (old, 'Single Uppset')
c                            '2' - Double (as before)
c
c 26-APR-1995  S P Cottrell  Modification to LF Abragam to give ZF form
c                            discussed by Keren.
c
c 18-Apr-1997 J.S.Lord	changes of function numbers for TF to match
c			corresponding LF ones.
c
c 28-Apr-1997 J.S.Lord	added Dynamic Kubo-Toyabe.
c
c 12-Feb-1998 J.S.Lord	upgraded ZF Abragam to general ZF/LF form (Keren)
c

c   function theory		makes the outher part of the job (N,Bgr.t0...)
c   functions mus1-mus6		makes the inner part ( a0*cos(wt+ph)*Gx )
c   function guessn0		guess value of n0, used for assymetry plots

c all routines in this file works with internal units, note however that
c user interaction (listing, input ..) users scaled units.
c internal units:
c   time:		us
c   phase:		radians
c   damping, frequency:	1/us (=MHz)
c   n, a0, bgr:	1
c n is the scaling parameter defined as number of detectable muon decay
c   events for a single pulse(frame) and a single telescope.
c   This correspons to the number we get out of the scalers as "pulses/frame".
c   If histograms are summed up in the grouping process we include the
c   factor 'histograms', defined as the number of histograms added together to
c   produce a group.
c N0, the traditional (unnormalized [counts/chan]) scaling parameter,
c  is then given as:
c  N0= n*histograms*frames*res/tmuon
c Adding new model functions:
c  New functions must be written as functions to be called by 'theory', and
c  the corresponding call must be entered into 'theory.
c  The program must then be compiled and UDA re-linked.
c  One must also prepare a PDF-file for the new theory to define its parameters.
c  From V.13 add the function to MUFUNC and add a new line to the file
c  PDF.UDA defining the parameters.
c  For more information consult a copy of: "UDA, the book" or look in help
c  library.

	options /check=nounderflow
	real*8 function theory(t)
	include 'inc.uda'
	include 'com.dat'
	include 'com.fit'
	real t

c some numerical constants
	real tmuon	!the muon lifetime [us]
	real tpulse	!time between the proton pulses [us]
	parameter(tmuon=2.19714, tpulse=0.33)

c lineshape functions:
c	real*8 mus1,mus2,mus3,mus4,mus5,mus6,muzf
c	real a0max,fn0
	real*8 mufunc

c theory parameter
	real t0
	real corr
	real bgr
	real n
	real bal,fn0

c local storage
	real n0
	real tc,tc1,tc2	!for t-t0
	real*8 f,g,g1,g2

c parameters for mus
c	integer lsp,lsp2,lsp3,pfirst,ncp
	integer lsp,ncp
	real gw


c simpel (transversal) grouping, fth=1
c theory parameters (all other parameters are passed to the 'mus' function
c  par(1) t0	zero time, time when muon pulse arrived (or midpoint of pulses)
c  par(2) corr	dead-time
c  par(3) bgr	constant background
c  par(5) n	scaling param=number of events/frame/histogram

c calculation flow (tc, g and f are temporary variables)
c Initial step, Single pulse:
c  tc=t-t0
c  g=musX(tc)
c  f=n0*exp(tc/tmuon)*(1+g)+bgr
c Initial step, Double pulse
c  tc1=t-t0+tpulse/2
c  tc2=t-t0-tpulse/2
c  g1=mus(tc1)
c  g2=mus(tc2)
c Final step
c  n0=n*frames*res/tmuon
c  f=0.5*n0*(exp(tc1/tmuon)*(1+g1)+exp(tc2/tmuon)*(1+g2))+bgr

c we apply corerction according to OlaH, a more correct treatment should
c be substituted for
c  theory=f, if corr=0 (no dead-time correction)
c  theory=n0/corr*(1-exp(-f*corr/n0), if corr is used

c ----- treat transverse and longitudinal in the same way as long
c -----  as possible   SEP-92

c put the variables used by theory into variables with reasonable names
	  t0=  par(1)
	  corr=par(2)

! for historical reasons t0 for a single pulse has '0.5*tpulse' subtracted
	if(fpulse.eq.0) then          ! single pulse
	     tc = t-(t0-0.5*tpulse)   ! time from trigger to pulse shorter
	else if(fpulse.eq.1) then     ! average run, one pulse only
	     tc=t-t0
	     if(tc.lt.0.0 .and.fth.eq.2) tc=0.0	! F-B asym should really be
						! 'undefined' before t0, but
						! asym(t0) is better than 0
	else if(fpulse.eq.2) then
	     tc1=t-t0+tpulse/2.0      ! time from first pulse
	     if(tc1.lt.0.0 .and.fth.eq.2) tc1=0.0 ! F-B asym
	     tc2=t-t0-tpulse/2.0      ! time from second pulse
             gw= exp(tpulse/tmuon)    ! additional weigth for second pulse
	endif

c call musXXX to calculate 'g', according to the lineshape flag, flsh
c  musXX replaced by general functions MUTF and MUZF  SEP-92
        if (flsh.lt.20) ncomp=1
        if (flsh.ge.20.and.flsh.lt.400) ncomp=2
        if (flsh.gt.400) ncomp=3
        g = 0
        g1=0
        g2 = 0

        do ncp = 1, ncomp
            if (ncp.eq.1) lsp=lsp1
            if (ncp.eq.2) lsp=lsp2
            if (ncp.eq.3) lsp=lsp3
	    if(fpulse.le.1) then      ! single or average pulse
                if (tc.ge.0) g = g + mufunc (tc, lsp, par(pfirst(ncp)))
	    else if(fpulse.eq.2) then
                if (tc1.ge.0) g1 = g1 + mufunc (tc1, lsp, par(pfirst(ncp)))
                if (tc2.ge.0) g2 = g2 + mufunc (tc2, lsp, par(pfirst(ncp)))
	    endif
        enddo          !    end loop over ncomp components 

c compute N0, for the TF geometry case,  if zero return here
        if (fth.eq.1) then
	  bgr= par(3)
	  n=  par(4)
 	    N0=histograms*frames/tmuon*res*n
	    if(N0.eq.0.0) then
	      theory=0.0
	      return
	    endif
            f = bgr
	    if (fpulse.le.1) then
	      if (tc.ge.0) f = f + N0*exp(-tc/tmuon)*(1.0+g)
	    else if(fpulse.eq.2) then
	      if (tc1.ge.0) f = f + 0.5*N0*exp(-tc1/tmuon)*(1.0+g1)
              if (tc2.ge.0) f = f+ 0.5*N0*exp(-tc2/tmuon)*(1.0+g2)
	    endif

	    if(corr.eq.0.0) then
	      theory=f
	    else
	      theory=n0/corr*(1-exp(-f*corr/n0)) !Ola/Roberto
	    endif
        endif    ! end fth=1  TF grouping
	    

c simpel F-B grouping (longitudinal)
c theory parameters (all following parameters are passed to the 'mus' function
c  par(1) t0   zero time, time when muon pulse arrived (or midpoint two pulses)
c  par(3) bal  balance, correct for giving incorrect alphas for grouping of data
c    par (2) = corr  as before

c calculation flow: (tc and g are temporary variables)
c  theory=g, if bal=0 (no unbalance correction), else:
c  theory= (2*g-bal)/(2.0+bal)	!as told by M.K., never checked this formula

	if (fth.eq.2) then
	  bal= par(3)
 
c    put in a type of correction ----- OH may-92
c     create an artificial decay spectrum  

	  if (fpulse.le.1) f=g
	  if (fpulse.eq.2) f= 0.5*(g1 + gw*g2)  ! add weight for second pulse
	  if(corr.eq.0.0) then
	    theory=f
	  else
            fn0 = n0gr(1)    ! created in UDAGROUP.FOR  SEP-92
	    g= fn0*exp(-tc/tmuon)*(1.0+f)
	    f = (fn0/corr)*(1-exp(-g*corr/fn0)) !Ola/Roberto
	    theory = f*exp(+tc/tmuon)/fn0 -1.
	  endif
          g = theory
c final step, apply unbalance correction if in use
	  if(bal.eq.0.0) then
	    theory=g
	  else
	    theory= (2*g-bal)/(2.0+bal*g)	!M.K. never checked formula
						!SFJC added *g
	  endif
	endif       !  end fth=2  long.grouping

	return
	end !theory


c MUFUNC ======= general TF and ZF geometry function(s)  SEP-92

	options /check=nounderflow
	real*8 function mufunc (t, lsp, p)
	implicit none
        integer lsp     !lineshape  
	real t		!time [us]

        real tc    ! tauc [us]
	real p(*)	!parameters
        real*8 a0, a0c,  sig, tmp, wl, br, gtt
        real twopi, fkt2,fabr
        parameter (twopi=2*3.141593)
	real getdkt ! function for Dynamic K-T


        if (lsp.ge.11 .and. lsp .le. 14) then
c transverse field functions
c p(1)=frequency, p(2)=phase, p(3)=asym, p(4)=sigma/lambda, p(5)=tau/beta
          a0c = p(3)*cos(twopi*p(1)*t+p(2))
          tmp = p(4)*t ! sigma*t or lam*t
          if (lsp.eq.11) mufunc = a0c*exp(-tmp)
	  if (lsp.eq.12) mufunc = a0c*exp(-tmp**2)
c safe stretched exp
	  if (lsp.eq.13) mufunc = a0c*exp(-sign(abs(tmp)**p(5),tmp))
          if (lsp.eq.14) mufunc = a0c*
     ! exp(-(p(4)*p(5))**2*(exp(-t/p(5))-1.0+t/p(5)))      ! Abragam
	  return
        endif
c LF and ZF functions
	if(lsp .ge. 1 .and. lsp .le. 7) then
            a0 = p(1)
            sig = p(2)
            tmp = t*sig
            tc = p(3)
	    if (lsp.eq.1) mufunc = a0*exp(-tmp)
            if (lsp.eq.2) mufunc = a0*exp(-tmp*tmp)
c stretched exp: safe if tried with lambda<0 and beta non-integral
            if (lsp.eq.3) mufunc = a0*exp(-sign(abs(tmp)**tc,tmp))
c           if (lsp.eq.4) mufunc =a0*exp(-2.*(sig*tc)**2*(exp(-t/tc)-1.0+t/tc))
c	Abragam function in ZF or LF
c	(A. Keren, Phys Rev B vol.50 (1994) pp10039-10042)
	    if (lsp.eq.4) then
	      if(tc.lt.1.0E-6) tc=1.0E-6 ! prevent overflow ???
	      wl=p(4)*1.355342E-2*twopi ! Larmor freq in B_ext
	      br=(wl**2+(1/tc)**2)*t/tc
     +		+(wl**2-(1/tc)**2)*(1-exp(-t/tc)*cos(wl*t))
     +		-2./tc*wl*exp(-t/tc)*sin(wl*t)
	      gtt=2*sig**2*br/((wl**2+(1/tc)**2)**2)
	      mufunc=a0*exp(-gtt)
	    endif
            if (lsp.eq.5) then    !  Kubo-Toyabe (Gaussian)
                tmp= tmp**2       !gamma*delta[1/us]*t[us] all squared
	        mufunc = a0*(0.3333+0.6667*(1-tmp)*exp(-0.5*tmp))
            endif       
            if (lsp.eq.6) then    !  Kubo-Toyabe (Lorentzian)
	        mufunc = a0*(0.3333+0.6667*(1-tmp)*exp(-tmp))
            endif       
            if (lsp.eq.7) then 
                tmp= tmp**2       !   Take Kubo-Toyabe
	        fkt2 = 0.6667*(1-tmp)*exp(-0.5*tmp)
                fabr = exp(-(sig*tc)**2*(exp(-t/tc)-1.0+t/tc))      ! Abragam
                mufunc = a0*0.3333 + a0*fkt2*(fabr/exp(-tmp))  ! Fake Hay
c                  try to multiply by 'noramlized' abragam
            endif
	    return
	endif
	if(lsp .eq.8) then ! Dynamic Kubo_toyabe, p2=sigma, p3=nu
	  mufunc=getdkt(t,p(2),p(3))*p(1)
	  return
	endif
	write(*,*)'MUFUNC called with lineshape ',lsp ! debug
	mufunc=0.0 ! if otherwise undefined
	return
        end  ! mufunc

c GUESSN0, makes a (linear) least squares fit to f=N0*exp(-t/2.197)
c if double pule use f=0.5*N0*( exp(-(t+0.165)/2.197)+exp(-(t-0.165)/2.197) )
c returns N0 in units of [ev]
c TODO speed up (sum exp analytically)
c TODO include t0 and bgr
	real function guessn0(nch,data,time)
	include 'inc.uda'
	integer nch,ic
	real data(nch)	!data N(t)
	real time(nch)	!time points t
	real t,sdf,sf2
	include 'com.fit'
	sdf=0.0
	sf2=0.0
	if(fth.eq.2) then	!double pulse function
	  do ic=1,nch
	    t=time(ic)-par(2)
	    sdf=sdf+data(ic)*0.5*(exp(-(t+0.165)/2.197)+exp(-(t-0.165)/2.197))
	    sf2=sf2+(0.5*(exp(-(t+0.165)/2.197)+exp(-(t-0.165)/2.197)))**2
	  end do
	else
	  do ic=1,nch
	    t=time(ic)-par(2)
	    sdf=sdf+data(ic)*exp(-t/2.197)
	    sf2=sf2+exp(-2.0*t/2.197)
	  end do
	endif
	if(sf2.gt.0.1) then
	  guessn0=sdf/sf2
	else
	  guessn0=0.0
	endif
	end !guessn0

c===============================================
c dyamic K-T function
c based on code for:
c	Dynamic f-u-f relaxation function
c	from Brewer et al, Hyperf. Int. 32 (1986) pp677-682
	real function getdkt(t,sig,nu)
	implicit none
	real t,sig,nu
C
	integer tsmax,stk
	parameter (tsmax=2000) ! 16 us of valid data
	parameter (stk=25)
	double precision eps
	parameter (eps=0.008) ! "bin" for calculations
	integer k,j,sp,ls
	double precision gs(tsmax),gd(tsmax,stk)
	double precision y,tt,hop
	real xe,oldsig(stk),oldnu(stk)
	integer xi
	data ls /0/, oldsig /stk * -999.0/ 

	save /gd/,/olda2/,/olda3/,/ls/

c	cc=cc+1
c	if(cc .gt. 10000) then
c	  cc=1
c	  write(*,*)'10000 calls'
c	endif

	do 5 sp=1,stk
	  if( sig .eq. oldsig(sp) .and. nu .eq. oldnu(sp)) goto 66
5	continue

c have to recalculate a table
c	rc=rc+1
c	if(rc .gt. 20) then
c	  rc=1
c	  write(*,*)'20 recalcs'
c	endif

	ls=ls+1
	if (ls .gt. stk .or. ls .lt. 1) then
	  ls=1
	endif
	sp=ls

	oldsig(ls)=sig
	oldnu(ls)=nu

	hop=nu*eps

c generate static f-u-f
c	do 10 k=1,tsmax
c	  t=(k-1)*tstep
c	  gs(k)=(3.+cos(sqrt(3.)*t)
c     1		+ (1-1/sqrt(3.))*cos((3.-sqrt(3.))/2*t)
c     2		+ (1+1/sqrt(3.))*cos((3.+sqrt(3.))/2*t) )/6.
c10	continue

c generate static Kubo-Toyabe
	do 10 k=1,tsmax
	  tt=((k-1)*eps*sig)**2
	  gs(k)=0.3333333333+0.6666666667*(1-tt)*exp(-tt/2)
10	continue
	

c generate dynamic f-u-f
	  do 30 k=1,tsmax
	    y=gs(k)
	    do 40 j=k-1,2,-1
	      y=y*(1-hop)+hop*gd(k-j+1,ls)*gs(j)
40	    continue
	    gd(k,ls)=y
30	  continue

66	continue

c interpolate table. If beyond end, extrapolate...
	XI=int(abs(t)/eps)
	if(xi .gt. tsmax-2) xi=tsmax-2
	xe=abs(t)/eps-xi
	
	getdkt=gd(xi+1,sp)*(1-xe)+xe*gd(xi+2,sp)
	return
	end

